@@ -326,6 +326,10 @@ sub log
     } else {
         print STDERR strftime("%Y %m %d %H:%M:%S", localtime(time))." $p{'message'}\n";
     }
+
+    if (defined($conf->{printlog}) && $conf->{printlog} =~ /$p{'level'}/) {
+        print $p{'message'}."\n";
+    }
 }
 
 sub _checkConfig
@@ -460,7 +464,7 @@ sub _searchLism
             $filter = '(objectClass=*)';
         }
 
-        my ($rc, @entries) = $self->{lism}->search($base, $scope, 0, 0, 0, encode('utf8', $filter), 0, $attr);
+        my ($rc, @entries) = $self->{lism}->search($base, $scope, 0, 0, 0, encode('utf8', $filter), 0, $attr, defined($lismopts->{attrs}) ? @{$lismopts->{attrs}} : 'objectClass');
         if ($rc) {
             $self->log(level => 'err', message => "lism search by $filter failed($rc)");
 
@@ -476,7 +480,7 @@ sub _searchLism
 
                 my @tmpvals;
                 if ($lismopts->{attr} eq 'dn') {
-                    @tmpvals = ($entryStr =~ /^[^\n]+/);
+                    @tmpvals = ($entryStr =~ /^dn: ([^\n]+)/);
                 } elsif ($lismopts->{attr} eq 'parentrdn') {
                     @tmpvals = ($entryStr =~ /^[^,]+,([^,]+)/);
                 } else {
@@ -516,6 +520,16 @@ sub _unique
     return @uarray;
 }
 
+sub _delquote
+{
+    my ($str) = @_;
+
+    $str =~ s/^["']*//;
+    $str =~ s/["']*$//;
+
+    return $str;
+}
+
 sub lock
 {
     my $self = shift;
