@@ -3,6 +3,7 @@ package LISM::Handler::Check;
 use strict;
 use base qw(LISM::Handler);
 use POSIX qw(strftime);
+use Config::IniFiles;
 use Encode;
 use Net::LDAP::Constant qw(:all);
 use Data::Dumper;
@@ -224,6 +225,17 @@ sub _checkValues
                 }
             }
         }
+        my %params;
+        if ($type ne 'post') {
+            if (defined($rule->{param})) {
+                foreach my $param (keys(%{$rule->{param}})) {
+                   my $value = $self->_getParam($rule->{param}{$param}, $dn, $param);
+                   if (defined($value)) {
+                       $params{$param} = $value;
+                   }
+                }
+            }
+        }
         foreach my $attr (keys %{$rule->{attr}}) {
             my $cattr = $rule->{attr}{$attr};
             my @values = ($entryStr =~ /^$attr: (.+)$/gmi);
@@ -304,9 +316,13 @@ sub _checkValues
                     $self->_perror("$attr=$value in $dn is too long");
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                 }
-                if (defined($cattr->{regexp}) && $value !~ /$cattr->{regexp}[0]/i) {
-                    $self->_perror("$attr=$value in $dn is invalid: regular expression is $cattr->{regexp}[0]");
-                    $rc = LDAP_CONSTRAINT_VIOLATION;
+                if (defined($cattr->{regexp})) {
+                    my $regexp = $cattr->{regexp}[0];
+                    $regexp = $self->_replaceParam($regexp, %params);
+                    if ($value !~ /$regexp/i) {
+                        $self->_perror("$attr=$value in $dn is invalid: regular expression is $regexp");
+                        $rc = LDAP_CONSTRAINT_VIOLATION;
+                    }
                 }
                 if (defined($cattr->{function})) {
                     my $ecode = 0;
@@ -342,6 +358,15 @@ sub _checkValues
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
+                if (defined($cattr->{pwdpolicy})) {
+                    my ($prc, $error) = $self->_checkPwdPolicy($cattr->{pwdpolicy}[0], $dn, $oldentry, $value);
+                    if (!defined($prc)) {
+                        return LDAP_OTHER;
+                    } elsif (!$prc) { 
+                        $self->_perror("$attr=$value in $dn is invalid: $error");
+                        $rc = LDAP_CONSTRAINT_VIOLATION;
+                    }
+                }
             }
         }
     }
@@ -349,6 +374,174 @@ sub _checkValues
     return $rc;
 }
 
+sub _replaceParam
+{
+    my $self = shift;
+    my ($str, %params) = @_;
+
+    my @keys = ($str =~ /\%{([^}]+)}/g);
+    foreach my $key (@keys) {
+        my $value = defined($params{$key}) ? $params{$key} : '';
+        $str =~ s/\%{$key}/$value/g;
+    }
+
+    return $str;
+}
+
+sub _getParam
+{
+    my $self = shift;
+    my ($opts, $dn, $param) = @_;
+
+    my ($base) = ($dn =~ /($opts->{base})/i);
+    if (!$base) {
+        return 1;
+    }
+
+    my %params;
+    if (defined($self->{lism}->{bind}{check_param}{$base})) {
+        %params = %{$self->{lism}->{bind}{check_param}{$base}};
+    }
+    if (!%params || !defined($params{$param})) {
+        my $filter = defined($opts->{filter}) ? $opts->{filter} : '(objectClass=*)';
+        my ($rc, @entries) = $self->{lism}->search($base, 2, 0, 0, 0, $filter, 0);
+        if ($rc) {
+            $self->log(level => 'err', message => "searching parameter($base) failed($rc)");
+            return undef;
+        }
+
+        if (@entries) {
+            my @values;
+            foreach my $attr (split(/, */, $opts->{attr})) {
+                my @tmpvals = ($entries[0] =~ /^$attr: (.*)$/gmi);
+                if (@tmpvals) {
+                    push(@values, @tmpvals);
+                }
+            }
+            if (@values) {
+                if (@values > 1) {
+                    $params{$param} = '('.join('|', @values).')';
+                } else {
+                    $params{$param} = $values[0];
+                }
+            }
+        }
+        if (!defined($params{$param})) {
+            $params{$param} = defined($opts->{default}) ? $opts->{default} : '';
+        }
+        if (!defined($self->{lism}->{bind}{check_param})) {
+            $self->{lism}->{bind}{check_param} = {};
+        }
+        $self->{lism}->{bind}{check_param}{$base} = \%params;
+    }
+
+    return $params{$param};
+}
+
+sub _checkPwdPolicy
+{
+    my $self = shift;
+    my ($opts, $dn, $oldentry, $value) = @_;
+
+    my ($base) = ($dn =~ /($opts->{base})/i);
+    if (!$base) {
+        return 1;
+    }
+
+    my %pwdpolicy;
+    if (defined($self->{lism}->{bind}{pwdpolicy}{$base})) {
+        %pwdpolicy = %{$self->{lism}->{bind}{pwdpolicy}{$base}};
+    } else {
+        my $filter = defined($opts->{filter}) ? $opts->{filter} : '(objectClass=*)';
+        my ($rc, @entries) = $self->{lism}->search($base, 2, 0, 0, 0, $filter, 0);
+        if ($rc) {
+            $self->log(level => 'err', message => "searching password policy($base) failed($rc)");
+            return undef;
+        }
+
+        if (@entries) {
+            ($pwdpolicy{minlen}) = ($entries[0] =~ /^pwdMinLength: (.*)$/mi);
+            ($pwdpolicy{maxlen}) = ($entries[0] =~ /^seciossPwdMaxLength: (.*)$/mi);
+            ($pwdpolicy{inhistory}) = ($entries[0] =~ /^pwdInHistory: (.*)$/mi);
+            my @allowedchars = ($entries[0] =~ /^seciossPwdAllowedChars: (.+)$/gmi);
+            if (@allowedchars) {
+                $pwdpolicy{allowedchars} = \@allowedchars;
+            }
+            my @deniedchars = ($entries[0] =~ /^seciossPwdDeniedChars: (.+)$/gmi);
+            if (@deniedchars) {
+                $pwdpolicy{deniedchars} = \@deniedchars;
+            }
+            if (!defined($self->{lism}->{bind}{pwdpolicy})) {
+                $self->{lism}->{bind}{pwdpolicy} = {};
+            }
+            $self->{lism}->{bind}{pwdpolicy}{$base} = \%pwdpolicy;
+        } elsif (defined($opts->{file})) {
+            my $pwdconfig = Config::IniFiles->new(-file => $opts->{file});
+            if ($pwdconfig) {
+                $pwdpolicy{minlen} = LISM::Handler::_delquote($pwdconfig->val('password', 'pwminlen'));
+                $pwdpolicy{maxlen} = LISM::Handler::_delquote($pwdconfig->val('password', 'pwmaxlen'));
+                $pwdpolicy{inhistory} = LISM::Handler::_delquote($pwdconfig->val('password', 'pwinhistory'));
+                my $tmpval = $pwdconfig->val('password', 'pwallow');
+                my @allowedchars;
+                if (defined($tmpval)) {
+                    @allowedchars = ($tmpval);
+                } else {
+                    @allowedchars = $pwdconfig->val('password', 'pwallow[]');
+                }
+                if (@allowedchars) {
+                    for (my $i = 0; $i < @allowedchars; $i++) {
+                        $allowedchars[$i] = LISM::Handler::_delquote($allowedchars[$i]);
+                    }
+                    $pwdpolicy{allowedchars} = \@allowedchars;
+                }
+                $tmpval = $pwdconfig->val('password', 'pwdeny');
+                my @deniedchars;
+                if (defined($tmpval)) {
+                    @deniedchars = ($tmpval);
+                } else {
+                    @deniedchars = $pwdconfig->val('password', 'pwdeny[]');
+                }
+                if (@deniedchars) {
+                    for (my $i = 0; $i < @deniedchars; $i++) {
+                        $deniedchars[$i] = LISM::Handler::_delquote($deniedchars[$i]);
+                    }
+                    $pwdpolicy{deniedchars} = \@deniedchars;
+                }
+                if (!defined($self->{lism}->{bind}{pwdpolicy})) {
+                    $self->{lism}->{bind}{pwdpolicy} = {};
+                }
+                $self->{lism}->{bind}{pwdpolicy}{$base} = \%pwdpolicy;
+            }
+        }
+    }
+
+    if ($pwdpolicy{minlen} && $pwdpolicy{minlen} > length($value)) {
+        return (0, "password length is too short");
+    }
+    if ($pwdpolicy{maxlen} && $pwdpolicy{maxlen} < length($value)) {
+        return (0, "password length is too long");
+    }
+    if ($pwdpolicy{inhistory} && $oldentry && grep(/^$value$/, ($oldentry =~ /^seciossPwdHistory: (.+)$/))) {
+        return (0, "password is in the history");
+    }
+    if (defined($pwdpolicy{allowedchars})) {
+        foreach my $chars (@{$pwdpolicy{allowedchars}}) {
+            if ($chars !~ /^ *$/ && $value !~ /$chars/) {
+                return (0, "password characters are invalid");
+            }
+        }
+    }
+    if (defined($pwdpolicy{deniedchars})) {
+        foreach my $chars (@{$pwdpolicy{deniedchars}}) {
+            if ($chars !~ /^ *$/ && $value =~ /$chars/) {
+                return (0, "password characters are invalid");
+            }
+        }
+    }
+
+    return 1;
+}
+
 sub _checkMaxEntries
 {
     my $self = shift;
@@ -593,7 +786,12 @@ sub _checkConfig
                     if (defined($rule->{attr}{$cattr}->{lismopts})) {
                         next;
                     }
-                    my ($name) = keys %{$rule->{attr}{$cattr}};
+                    my $name;
+                    foreach my $key (keys %{$rule->{attr}{$cattr}}) {
+                        if ($key =~ /^lismexist|lismunique|valexists$/) {
+                            $name = $key;
+                        }
+                    }
                     my $lismopts = {};
                     my ($base, $attr, $scope, $filter, $option) = split(/\?/, $rule->{attr}{$cattr}->{$name}[0]);
                     $lismopts->{base} = $base;
