@@ -2,6 +2,7 @@ package LISM::Handler::Rewrite;
 
 use strict;
 use base qw(LISM::Handler);
+use Net::LDAP::Constant qw(:all);
 use Net::LDAP::Filter;
 use Encode;
 use LISM::Storage;
@@ -43,13 +44,13 @@ sub pre_bind
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$binddnp});
             if (!$str) {
                 $self->log(level => 'err', message => "bind rewrite \"${$binddnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             (${$binddnp}) = split(/\n/, $str);
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
 =head2 pre_compare($dnp, $avaStrp)
@@ -77,20 +78,20 @@ sub pre_compare
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
                 $self->log(level => 'err', message => "compare rewrite \"${$dnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
 
             $str = $self->_rewriteParse($rule->{match}, $substitution, ${$avaStrp}, \%rwcache);
             if (!$str) {
                 $self->log(level => 'err', message => "compcare rewrite \"${$dnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
              }
              (${$avaStrp}) = split(/\n/, $str);
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
 =head2 pre_search($basep, $filterStrp)
@@ -118,7 +119,7 @@ sub pre_search
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$basep}, \%rwcache);
             if (!defined($str)) {
                 $self->log(level => 'err', message => "search rewrite \"${$basep}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             (${$basep}) = split(/\n/, $str);
 
@@ -127,7 +128,7 @@ sub pre_search
                 $str = $self->_rewriteParse($rule->{match}, $substitution, $elts[$i], \%rwcache);
                 if (!defined($str)) {
                     $self->log(level => 'err', message => "search rewrite \"${$filterStrp}\" failed");
-                   return -1;
+                   return LDAP_OPERATIONS_ERROR;
                 }
 
                 my $elt;
@@ -145,10 +146,10 @@ sub pre_search
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
-=head2 post_search($entriesp)
+=head2 post_search($entriesp, $attrsp)
 
 Rewrite search results.
 
@@ -157,11 +158,15 @@ Rewrite search results.
 sub post_search
 {
     my $self = shift;
-    my ($entriesp) = @_;
+    my ($entriesp, $attrsp) = @_;
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'searchResult') {
+            if (defined($rule->{attrs}) && @{$attrsp} && !grep(/$rule->{attrs}/i, @{$attrsp})) {
+                next;
+            }
+
             my %rwcache;
             for (my $i = 0; $i < @{$entriesp}; $i++) {
                 my $entryStr = ${$entriesp}[$i];
@@ -203,7 +208,7 @@ sub post_search
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
 =pod
@@ -234,7 +239,7 @@ sub pre_modify
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
                 $self->log(level => 'err', message => "modify rewrite \"${$dnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
 
@@ -258,7 +263,7 @@ sub pre_modify
                         $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".$values[$i], \%rwcache);
                         if (!defined($str)) {
                             $self->log(level => 'err', message => "modify rewrite \"${$dnp}\" failed");
-                            return -1;
+                            return LDAP_OPERATIONS_ERROR;
                         } elsif ($str) {
                             foreach my $line (split(/\n/, $str)) {
                                 my ($rwattr, $value) = ($line =~ /(^[^:]*): (.*)$/);
@@ -280,7 +285,7 @@ sub pre_modify
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
 =pod
@@ -302,7 +307,7 @@ sub pre_add
             if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
                 next;
             }
-            if (defined($rule->{filter}) && !LISM::Storage->parseFilter($rule->{filterobj}, "${$dnp}\n${$entryStrp}")) {
+            if (defined($rule->{filter}) && !LISM::Storage->parseFilter($rule->{filterobj}, "${$dnp}\n${$entryStrp}[0]")) {
                 next;
             }
 
@@ -313,7 +318,7 @@ sub pre_add
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
                 $self->log(level => 'err', message => "add rewrite \"${$dnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
 
@@ -323,14 +328,14 @@ sub pre_add
                 $line[$i] = $self->_rewriteParse($rule->{match}, $substitution, $line[$i], \%rwcache);
                 if (!defined($line[$i])) {
                     $self->log(level => 'err', message => "add rewrite \"${$dnp}\" failed");
-                    return -1;
+                    return LDAP_OPERATIONS_ERROR;
                 }
             }
             ${$entryStrp}[0] = join("\n", $self->_unique(@line))."\n";
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
 =head2 pre_modrdn($dnp, $argsp)
@@ -358,20 +363,20 @@ sub pre_modrdn
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
                 $self->log(level => 'err', message => "modrdn rewrite \"${$dnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
 
             $str = $self->_rewriteParse($rule->{match}, $substitution, ${$argsp}[0], \%rwcache);
             if (!$str) {
                 $self->log(level => 'err', message => "modrdn rewrite \"${$dnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             ${$argsp}[0] = $str;
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
 =pod
@@ -401,13 +406,13 @@ sub pre_delete
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
                 $self->log(level => 'err', message => "delete rewrite \"${$dnp}\" failed");
-                return -1;
+                return LDAP_OPERATIONS_ERROR;
             }
             (${$dnp}) = split(/\n/, $str);
         }
     }
 
-    return 0;
+    return LDAP_SUCCESS;
 }
 
 
@@ -442,7 +447,19 @@ sub _checkConfig
             } elsif ($type eq 'lism') {
                 if (!defined($self->{lismmap})) {$self->{lismmap} = {}};
                 my $lismmap = {};
-                ($lismmap->{base}, $lismmap->{attr}) = split(/\?/, $conf->{rewritemap}{$map_name}->{attrs});
+                my ($base, $attrsStr, $scope, $sizelimit) = split(/\?/, $conf->{rewritemap}{$map_name}->{attrs});
+                $lismmap->{base} = $base;
+                my ($attr, @attrs) = split(/,/, $attrsStr);
+                $lismmap->{attr} = $attr;
+                if (@attrs) {
+                    $lismmap->{attrs} = \@attrs;
+                }
+                if ($scope) {
+                    $lismmap->{scope} = $scope;
+                }
+                if ($sizelimit) {
+                    $lismmap->{sizelimit} = $sizelimit;
+                }
                 $self->{lismmap}{$map_name} = $lismmap;
             }
         }
@@ -451,12 +468,18 @@ sub _checkConfig
     if (defined($conf->{rewrite})) {
         foreach my $rule (@{$conf->{rewrite}}) {
             if (defined($rule->{filter})) {
-                $rule->{filterobj} = Net::LDAP::Filter->new($rule->{filter});
+                $rule->{filter} =~ s/&amp;/&/g;
+                $rule->{filterobj} = Net::LDAP::Filter->new(encode('utf8', $rule->{filter}));
+            }
+            if (defined($rule->{attrs})) {
+                $rule->{attrs} =~ s/,/|/;
+                $rule->{attrs} = '^('.$rule->{attrs}.')$';
             }
 
             # enable escape sequence
             $rule->{substitution} =~ s/([^\\])\\n/$1\n/g;
             $rule->{substitution} =~ s/([^\\])\\t/$1\t/g;
+            $rule->{substitution} =~ s/&amp;/&/g;
         }
     }
 
@@ -468,9 +491,23 @@ sub _rewritePattern
     my $self = shift;
     my ($str, $pattern, $value) = @_;
 
-    my ($qt) = ($str =~ /(['"])$pattern/);
-    if ($str =~ /$qt$pattern$qt/) {
-        $value =~ s/$qt/\\$qt/g;
+    my @rwmaps = ($str =~ /%{([^(]*\((?:(?!%{).)*\))}/gs);
+    foreach my $rwmap (@rwmaps) {
+        my $tmpstr = $rwmap;
+        my $tmpval = $value;
+        my $qt = '';
+        if ($rwmap =~ /'[^']*$pattern[^']*'/) {
+            $qt = '\'';
+        } elsif ($rwmap =~ /"[^"]*$pattern[^"]*"/) {
+            $qt = '"';
+        }
+
+        if ($qt) {
+            $tmpval =~ s/$qt/\\$qt/g;
+            $tmpstr =~ s/$pattern/$tmpval/g;
+        }
+        $rwmap =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
+        $str =~ s/%{$rwmap}/%{$tmpstr}/;
     }
 
     $str =~ s/$pattern/$value/g;
@@ -515,14 +552,14 @@ sub _rewriteParse
                 }
 
                 @values = $self->_rewriteMap($map_name, $map_args);
-                if (!defined(@values)) {
+                if (!defined($values[0])) {
                     return undef;
                 }
 
                 ${$rwcache}{$key} = \@values;
             }
 
-            if (!$values[0]) {
+            if ($values[0] eq '') {
                 return $str;
             }
 
@@ -590,7 +627,6 @@ sub _passArgs
     if ($pos < 0) {
         return $pos;
     }
-    $pos;
 
     my $leftstr = substr($str, $pos + 1);
     my ($qtchar) = ($leftstr =~ /^ *(['"])/);
