@@ -586,6 +586,10 @@ sub log
     } else {
         print STDERR strftime("%Y %m %d %H:%M:%S", localtime(time))." $p{'message'}\n";
     }
+
+    if (defined($conf->{printlog}) && $conf->{printlog} =~ /$p{'level'}/) {
+        print $p{'message'}."\n";
+    }
 }
 
 =pod
