@@ -444,6 +444,27 @@ sub add
     }
 
     my $msg = $self->{ldap}->add($dn, attrs => [%attrs]);
+    if (defined($conf->{deleteflag}) && $msg->code == LDAP_ALREADY_EXISTS) {
+        foreach my $key (keys %{$conf->{deleteflag}}) {
+            my $deleteflag = $conf->{deleteflag}{$key};
+            if (!defined($deleteflag->{ovrfilter})) {
+                next;
+            }
+
+            my $delentry;
+            ($rc, $delentry) = $self->_do_search(undef, $dn, 0, 0, 1, 0, $deleteflag->{ovrfilter}, 0, 'objectClass');
+            if ($rc) {
+                last;
+            }
+            if ($delentry) {
+                my $delmsg = $self->{ldap}->delete($dn);
+                if (!$delmsg->code) {
+                    $msg = $self->{ldap}->add($dn, attrs => [%attrs]);
+                }
+                last;
+            }
+        }
+    }
 
     $self->_freeConnect($msg);
 
@@ -717,6 +738,9 @@ sub _checkConfig
                 $conf->{deleteflag}{$key}->{filter} =~ s/&amp;/&/g;
                 $conf->{deleteflag}{$key}->{filterobj} = Net::LDAP::Filter->new($conf->{deleteflag}{$key}->{filter});
             }
+            if ($conf->{deleteflag}{$key}->{ovrfilter}) {
+                $conf->{deleteflag}{$key}->{ovrfilter} =~ s/&amp;/&/g;
+            }
         }
     }
 
