@@ -164,8 +164,8 @@ sub search
     my $count;
     my $offset = 1;
     my $key = 'cn';
-    if (defined($conf->{control}) && defined($conf->{control}{vlv})) {
-        $count = $conf->{control}{vlv}->{count};
+    if (defined($conf->{control}) && defined($conf->{control}[0]->{vlv})) {
+        $count = $conf->{control}[0]->{vlv}[0]->{count};
     }
     if (grep(/^uid$/, @attrs)) {
         $key = 'uid';
