@@ -266,7 +266,12 @@ sub _objSearch
 
     DO: {
         # get data of the entries
-        my $sql = "select distinct $oconf->{table}[0].$oconf->{id}[0]->{column}[0]";
+        my $sql = "";
+        if (defined($oconf->{nodistinct}) && $oconf->{nodistinct}[0] eq "on") {
+            $sql = "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0]";
+        } else {
+            $sql = "select distinct $oconf->{table}[0].$oconf->{id}[0]->{column}[0]";
+        }
         foreach my $attr ('objectclass', keys %{$oconf->{attr}}) {
             if (!defined($oconf->{attr}{$attr}) || !defined($oconf->{attr}{$attr}->{column})) {
                 next;
@@ -289,8 +294,9 @@ sub _objSearch
             my $poconf = $self->{object}{$oconf->{container}[0]->{oname}[0]}->{conf};
 
             if ($oconf->{table}[0] ne $poconf->{table}[0]) {
+                my $idquote = !defined($poconf->{id}[0]->{type}) || $poconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
                 $from = "$from, $poconf->{table}[0]";
-                $where = "$where and $poconf->{table}[0].$poconf->{id}[0]->{column}[0] ".(defined($pkey) ? "= \'$pkey\'" : "is NULL");
+                $where = "$where and $poconf->{table}[0].$poconf->{id}[0]->{column}[0] ".(defined($pkey) ? "= $idquote$pkey$idquote" : "is NULL");
             }
 
             if (defined($oconf->{container}[0]->{fromtbls})) {
@@ -314,10 +320,12 @@ sub _objSearch
         }
 
         $sql = "$sql from $from where $where";
+        if (defined($oconf->{sort})) {
+            $sql .= " order by $oconf->{sort}[0]";
+        }
 
         # encode value
         $sql = encode($conf->{mbcode}[0], $sql);
-
         my $sth = $self->{db}->prepare($sql);
         if (!$sth->execute) {
             $self->log(level => 'err', message => "Searching by \"$sql\" failed: ".$sth->errstr);
@@ -480,6 +488,7 @@ sub _objModify
             last DO;
         }
 
+        my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
         while ( @list > 0 ) {
             my $action = shift @list;
             my $attr    = lc(shift @list);
@@ -521,6 +530,7 @@ sub _objModify
                 }
             }
 
+            my $quote = !defined($oconf->{attr}{$attr}->{type}) || $oconf->{attr}{$attr}->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
             if($action eq "ADD") {
                 # check whether the value already exists
                 my $vals_str = "(".join('|', @values).")";
@@ -543,7 +553,7 @@ sub _objModify
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = $quote$value$quote where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -588,7 +598,7 @@ sub _objModify
                             $value = join($oconf->{attr}{$attr}->{delim}[0], @new_vals);
                         }
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = $quote$value$quote where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -627,7 +637,7 @@ sub _objModify
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = $quote$value$quote where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -758,23 +768,24 @@ sub _objAdd
             $value =~ s/'/''/g;
             $value =~ s/\\/\\\\/g;
 
+            my $quote = !defined($oconf->{attr}{$attr}->{type}) || $oconf->{attr}{$attr}->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
             if (defined($oconf->{attr}{$attr}->{column})) {
                 if (defined($oconf->{attr}{$attr}->{delim})) {
                     my $match = 0;
                     for (my $i = 0; $i < @cols; $i++) {
                         if ($cols[$i] =~ /^$oconf->{attr}{$attr}->{column}[0]$/) {
-                            $values[$i] =~ s/^'(.+)'$/\'$1$oconf->{attr}{$attr}->{delim}[0]$value\'/;
+                            $values[$i] =~ s/^$quote(.+)$quote$/$quote$1$oconf->{attr}{$attr}->{delim}[0]$value$quote/;
                             $match = 1;
                             last;
                         }
                     }
                     if (!$match) {
                         push(@cols, $oconf->{attr}{$attr}->{column}[0]);
-                        push(@values, "\'$value\'");
+                        push(@values, $quote.$value.$quote);
                     }
                 } elsif (!grep(/^$oconf->{attr}{$attr}->{column}[0]$/, @cols)) {
                     push(@cols, $oconf->{attr}{$attr}->{column}[0]);
-                    push(@values, "\'$value\'");
+                    push(@values, $quote.$value.$quote);
                 }
             }
             if (defined($oconf->{attr}{$attr}->{addproc})) {
@@ -788,10 +799,11 @@ sub _objAdd
             my $value = $self->_getStaticValue($strginfo, $dn, $entryStr);
             push(@si_values, $value);
 
+            my $quote = !defined($strginfo->{type}) || $strginfo->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
             if (defined($strginfo->{column})) {
                 if (!grep(/^$strginfo->{column}[0]$/, @cols)) {
                     push(@cols, $strginfo->{column}[0]);
-                    push(@values, "\'$value\'");
+                    push(@values, $quote.$value.$quote);
                 }
             }
         }
@@ -984,8 +996,9 @@ sub _objDelete
                 $sth->finish;
             }
 
+            my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
             # delete the appropriate record from the object's table
-            $sql = "delete from $oconf->{table}[0] where $oconf->{id}[0]->{column}[0] = \'$key\'";
+            $sql = "delete from $oconf->{table}[0] where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
             $sth = $self->{db}->prepare($sql);
             if (!$sth->execute) {
                 $self->log(level => 'err', message => "Deleting entry by \"$sql\" failed: ".$sth->errstr);
@@ -1130,7 +1143,8 @@ sub _getParentRdn
         $from = "$from,$oconf->{container}[0]->{fromtbls}[0]";
     }
 
-    $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = \'$key\'";
+    my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
+    $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
     if (defined($oconf->{container}[0]->{joinwhere})) {
         if ($oconf->{container}[0]->{joinwhere}[0] =~ /\%c/) {
             my ($pidcol) = ($oconf->{container}[0]->{joinwhere}[0] =~ /([^ ]+) *= *'?\%c'?/);
@@ -1220,7 +1234,8 @@ sub _getAttrValues
         $where =~ s/\%o/$key/g;
         $where =~ s/\%p/$pkey/g;
     } else {
-        $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = \'$key\'";
+        my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number|text|boolean)$/i ? "'" : '';
+        $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             $where = "$where and $oconf->{attr}{$attr}->{joinwhere}[0]";
             $where =~ s/\%p/$pkey/g;
@@ -1256,7 +1271,7 @@ sub _getAttrValues
             $attrStr =~ s/ *,/,/g;
         } else{
             $data[0] =~ s/ *$//;
-            if ($data[0]) {
+            if ($data[0] | $data[0] eq '0') {
                 if ($data[0] =~ /\n/) {
                     $data[0] = encode_base64($data[0], '');
                     $attrStr = $attrStr.$attr.":: $data[0]\n";
@@ -1461,6 +1476,7 @@ sub _filter2sql
         if (!$self->_filter2sql($oconf, $args, $from, $where)) {
             return 0;
         }
+        ${$where} = "not ${$where}";
 
         return 1;
     }
@@ -1501,6 +1517,8 @@ sub _filter2sql
     $value =~ s/'/''/g;
     $value =~ s/\\/\\\\/g;
 
+    my $nml = !defined($oconf->{attr}{$attr}) || !defined($oconf->{attr}{$attr}->{type}) || $oconf->{attr}{$attr}->{type}[0] !~ /^(int|smallint|float|number|date|text|boolean)$/i ? 1 : 0;
+    my $quote = !defined($oconf->{attr}{$attr}) || !defined($oconf->{attr}{$attr}->{type}) || $oconf->{attr}{$attr}->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
     ${$from} = $oconf->{table}[0];
     if ($attr =~ /^objectclass$/i) {
         # check object class in filter
@@ -1515,7 +1533,7 @@ sub _filter2sql
         # attribute doesn't exit
         ${$where} = '1 = 0';
     } elsif (defined($oconf->{attr}{$attr}->{column})) {
-        ${$where} = "lower($oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0]) $type lower(\'$value\')";
+        ${$where} = ($nml ? "lower($oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0])" : "$oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0]")." $type ".($nml ? "lower($quote$value$quote)" : "$quote$value$quote");
     } elsif (defined($oconf->{attr}{$attr}->{oname})) {
         my $aconf = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]}->{conf};
 
@@ -1526,8 +1544,10 @@ sub _filter2sql
             ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
 
+        my $nml = !defined($aconf->{attr}{$aconf->{rdn}[0]}) || !defined($aconf->{attr}{$aconf->{rdn}[0]}->{type}) || $aconf->{attr}{$$aconf->{rdn}[0]}->{type}[0] !~ /^(int|smallint|float|number|date|text|boolean)$/i ? 1 : 0;
+        my $quote = !defined($aconf->{attr}{$aconf->{rdn}[0]}) || !defined($aconf->{attr}{$aconf->{rdn}[0]}->{type}) || $aconf->{attr}{$aconf->{rdn}[0]}->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
         ($value) = ($value =~ /^$aconf->{rdn}[0]=([^,]*)/i);
-        my $subquery = "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where lower($aconf->{table}[0].$aconf->{attr}{$aconf->{rdn}[0]}->{column}[0]) $type lower(\'$value\')";
+        my $subquery = "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where ".($nml ? "lower($aconf->{table}[0].$aconf->{attr}{$aconf->{rdn}[0]}->{column}[0])" : "$aconf->{table}[0].$aconf->{attr}{$aconf->{rdn}[0]}->{column}[0]")." $type ".($nml ? "lower($quote$value$quote)" : "$quote$value$quote");
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             $subquery = "$subquery and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
@@ -1536,21 +1556,22 @@ sub _filter2sql
         if (defined($aconf->{container}) && !defined($aconf->{container}[0]->{rdn})) {
             my $paconf = $self->{object}{$aconf->{container}[0]->{oname}[0]}->{conf};
 
-            $from = "$from, $paconf->{table}[0]";
+            ${$from} = "${$from}, $paconf->{table}[0]";
             if (defined($aconf->{container}[0]->{fromtbls})) {
-                $from = "$from, $oconf->{container}[0]->{fromtbls}[0]";
+                ${$from} = "${$from}, $oconf->{container}[0]->{fromtbls}[0]";
             }
 
             if (defined($aconf->{container}[0]->{joinwhere})) {
-                $where = "$where and $aconf->{container}[0]->{joinwhere}[0]";
+                ${$where} = "${$where} and $aconf->{container}[0]->{joinwhere}[0]";
             }
         }
+    } elsif (defined($oconf->{attr}{$attr}->{constant})) {
+        ${$where} = "lower(\'$oconf->{attr}{$attr}->{constant}[0]\') $type lower(\'$value\')";
     } else {
         if (defined($oconf->{attr}{$attr}->{fromtbls})) {
             ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
-
-        my $subquery =  "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where lower($oconf->{attr}{$attr}->{selexpr}[0]) $type lower(\'$value\')";
+        my $subquery =  "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where ".($nml ? "lower($oconf->{attr}{$attr}->{selexpr}[0])" : $oconf->{attr}{$attr}->{selexpr}[0])." $type ".($nml ? "lower($quote$value$quote)" : "$quote$value$quote");
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             $subquery = "$subquery and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
